from covid_shared.cli_tools.decorators import add_verbose_and_with_debugger, pass_run_metadata
from covid_shared.cli_tools.logging import (LOG_FORMATS, DEFAULT_LOG_MESSAGING_FORMAT, add_logging_sink,
                                            configure_logging_to_files, configure_logging_to_terminal)
from covid_shared.cli_tools.metadata import (Metadata, RunMetadata, YamlIOMixin, get_function_full_argument_mapping,
                                             monitor_application, update_with_previous_metadata)
from covid_shared.cli_tools.run_directory import (get_last_stage_directory, get_current_previous_version,
                                                  get_run_directory, make_links, make_run_directory, mark_best,
                                                  mark_best_explicit, mark_explicit, mark_latest, mark_latest_explicit,
                                                  mark_production, mark_production_explicit, move_link,
                                                  setup_directory_structure)



from pathlib import Path

import pytest

from covid_shared.shell_tools import mkdir


@pytest.fixture(params=range(0o700, 0o1000, 3))
def mode(request):
    return request.param


@pytest.fixture(params=[True, False])
def parents(request):
    return request.param


def test_mkdir_no_args(mode: int, tmp_path: Path):
    tmp_path.rmdir()
    assert not tmp_path.exists()
    mkdir(tmp_path, mode=mode)
    assert tmp_path.exists()
    assert oct(tmp_path.stat().st_mode)[-3:] == oct(mode)[-3:]


def test_mkdir_no_parents(mode: int, tmp_path: Path):
    tmp_path.rmdir()
    assert not tmp_path.exists()

    child_dir = tmp_path / 'child'
    with pytest.raises(FileNotFoundError):
        mkdir(child_dir, mode)


def test_mkdir_parents(mode: int, tmp_path: Path):
    tmp_path.rmdir()
    assert not tmp_path.exists()

    child_dir = tmp_path / 'child'
    mkdir(child_dir, mode, parents=True)
    assert tmp_path.exists()
    assert oct(tmp_path.stat().st_mode)[-3:] == oct(mode)[-3:]
    assert child_dir.exists()
    assert oct(child_dir.stat().st_mode)[-3:] == oct(mode)[-3:]


def test_mkdir_exists(mode: int, parents: bool, tmp_path: Path):
    assert tmp_path.exists()
    perms = oct(tmp_path.stat().st_mode)[-3:]

    with pytest.raises(FileExistsError):
        mkdir(tmp_path, mode, parents=parents)

    mkdir(tmp_path, mode, parents=parents, exists_ok=True)
    assert oct(tmp_path.stat().st_mode)[-3:] == perms
